
#include <stdio.h>
#include <string.h>
#include <d3dx8math.h>

#include "math_matrix.h"
#include "Vertex.h"
#include "Character.h"


static int modelCount = 0;
static int meshCount  = 0;
static int anicount   = 0;
static const int ClockIndex[3] = { 0, 2, 1 };
int frameCount = 0;

static FILE *fp;
static char line[1024];

LPDIRECT3DTEXTURE8 g_texture[MAXTEXTURE];

tMESH::tMESH()
{
	pos.x = 0;
	pos.y = 0;
	pos.z = 0;

	rot.x = 0;
	rot.y = 0; //-D3DX_PI/2;
	rot.z = 0;

	rotate = EulerToQuatMtr( rot );
}

C3PCharacter::C3PCharacter() 
{
	m_pd3dDevice = NULL;
	m_pVB		 = NULL;
	m_pVertex	 = NULL;
}

C3PCharacter::~C3PCharacter()
{
	Close();
}

void C3PCharacter::Close()
{
	SAFE_RELEASE( m_pVB );
}


//- Reference SMD Load ---------------------------------------------------------------------------

bool C3PCharacter::LoadRefSMD( char* fileName)
{
	char str[256];
	int  data;
	int  boneCount = 0;

	fp = fopen(fileName, "rt");

	if( fp == 0 )
	{
		MessageBox(NULL, "File Not Found", "error", MB_OK);

		return false;
	}

	while( !feof(fp))
	{
		fgets(line, sizeof(line), fp);
		sscanf(line, "%s %d", str, &data);

		if( _stricmp( str, "version") == 0 )
		{
			if( data != 1)
			{
				MessageBox(NULL, "Incorrect Version", "error", MB_OK);
				return false;
			}

			modelCount++;
		}
		else if( _stricmp( str, "nodes" ) == 0 )
		{
			ParseNodes( &m_ModelList[modelCount-1].boneList[boneCount]);
			
			boneCount++;
		}
		else if( _stricmp( str, "skeleton" ) == 0 )
		{
			ParseSkeleton( &m_ModelList[modelCount-1].boneList[0] );
		}
		else if( _stricmp( str, "triangles" ) == 0 )
		{
			ParseTriangle( &m_ModelList[modelCount-1].meshList[meshCount] );
		}

	}

	meshCount++;
	m_iNumModel++;

	fclose(fp);

	return true;
}


//- Animation SMD Load ---------------------------------------------------------------------------

bool C3PCharacter::LoadAniSMD( char* fileName )
{
	char str[256];
	int  data;

	fp = fopen( fileName, "rt" );
	
	if( fp == 0 )
	{
		MessageBox(NULL, "File Not Found", "error", MB_OK);
		return false;
	}

	while( !feof(fp) )
	{
		fgets(line, sizeof(line), fp);
		sscanf(line, "%s %d", str, &data);

		if( _stricmp( str, "version") == 0 )
		{
			if( data != 1)
			{
				MessageBox(NULL, "Incorrect Version", "error", MB_OK);
				return false;
			}
		}
		else if( _stricmp( str, "skeleton" ) == 0 )
		{
			ParseAnimation( &m_ModelList[modelCount-1].aniList[0] );
		}
		
	}

	fclose(fp);
	return true;

}


//- Functions for Parsing Reference SMD -------------------------------------------------------------------------

void C3PCharacter::ParseNodes( tBONE_NODE *pNodes )
{
	int index, parent, numBones = 0;
	char name[32];

	while( fgets(line, sizeof(line), fp) != NULL )
	{	
		if( sscanf(line, " %d \"%[^\"]\" %d", &index, name, &parent ) == 3 )
		{
			if( strlen( pNodes[index].node_name ) != 0 ) break; // ߺ Bone üũ  // Ҵ name    ִ ƴϸ pNodes  Ⱑ ִ üũ

			strcpy( pNodes[index].node_name, name );
			pNodes[index].parent = parent;
			numBones = index;
		}
		else
		{
			m_ModelList[modelCount-1].numOfBones = numBones + 1;
			return;
		}
	}
}

//- Functions for Parsing Skeleton -------------------------------------------------------------------------
void C3PCharacter::ParseSkeleton( tBONE_NODE *pNodes )
{
	int index;
	float x, y, z, rx, ry, rz;
	char cmd[32];

	while( fgets(line, sizeof(line), fp) != NULL )
	{
		if( sscanf( line, " %d %f %f %f %f %f %f", &index, &x, &y, &z, &rx, &ry, &rz ) == 7 )
		{
			pNodes[index].pos.x = x;
			pNodes[index].pos.y = y;
			pNodes[index].pos.z = z;

			pNodes[index].rot.x = rx;
			pNodes[index].rot.y = ry;
			pNodes[index].rot.z = rz;
		}
		else if( sscanf( line, "%s", cmd ) == 1 )
		{
			if( _stricmp( cmd, "end" ) == 0 ) 
			{
				SetMatrix( pNodes );
				return;
			}
		}
	}
}

void C3PCharacter::SetMatrix( tBONE_NODE *pBone )
{
	int			 numOfBones, parent;
	D3DXMATRIX   matrix;
	ANGLEVECTOR  temp, angle;

	numOfBones = m_ModelList[modelCount-1].numOfBones;

	for( int i = 0; i < numOfBones; i++ )
	{
		parent = pBone[i].parent;

		if( parent == -1 )
		{
			pBone[i].TM    = Angle2Matrix( pBone[i].rot );
			pBone[i].iTM   = *( D3DXMatrixTranspose( &pBone[i].iTM, &pBone[i].TM ) );
			pBone[i].m_pos = pBone[i].pos;
		}
		else
		{
			matrix         = Angle2Matrix( pBone[i].rot );
			pBone[i].TM    = MatrixMulti( pBone[parent].TM, matrix );

			matrix         = *( D3DXMatrixTranspose( &matrix, &matrix ) );
			pBone[i].iTM   = MatrixMulti( matrix, pBone[parent].iTM );

			temp		   = VectorTransform( pBone[i].pos, pBone[parent].TM );
			pBone[i].m_pos = temp + pBone[parent].m_pos;
		}
	}
}



//- Functions for Parsing Triangle -------------------------------------------------------------------------
void C3PCharacter::ParseTriangle( tMESH *pMesh )
{
	int textureIndex = 0;
	int index, triCount = 0, vCount = 0, textureCount = 0;
	float x, y, z, nx, ny, nz, tu, tv;
	char textureName[32];

	while( !feof(fp) )
	{
		fgets(line, sizeof(line), fp);
		sscanf(line, "%s", textureName);

		if( _stricmp( textureName, "end" ) == 0 )
		{
			break;
		}

		if( textureCount == 0 )
		{
			strcpy( pMesh->textureList[0], textureName );
			pMesh->numOfTextures = 1;
			textureCount++;
		}


		for( int i = 0; i < textureCount; i++ )
		{
			if( _stricmp( textureName, pMesh->textureList[i] ) == 0 )
			{
				textureIndex = i;
				break;
			}

			else if( i == textureCount-1 )
			{
				textureCount++;

				pMesh->numOfTextures = textureCount;
				textureIndex = textureCount - 1;

				strcpy( pMesh->textureList[textureIndex], textureName );

				break;
			}
		}

		pMesh->triangleList[triCount].textureIndex = textureIndex;

		// ﰢ Ÿ parsing...-,-;
		D3DXVECTOR3 temp;
		tVERTEX vert;
		
		triCount++;
		
		for( i = 0; i < 3; i++ )
		{
			fgets(line, sizeof(line), fp);

			if( sscanf( line, "%d %f %f %f %f %f %f %f %f", &index, &x, &y, &z, &nx, &ny, &nz, &tu, &tv ) == 9 )
			{
				pMesh->triangleList[triCount-1].Tri[i].boneIndex = index;	// θ Boneε
				pMesh->triangleList[triCount-1].Tri[i].pos.x = x;			// ġ
				pMesh->triangleList[triCount-1].Tri[i].pos.y = y;
				pMesh->triangleList[triCount-1].Tri[i].pos.z = z;
				pMesh->triangleList[triCount-1].Tri[i].normal.x = nx;		// 븻
				pMesh->triangleList[triCount-1].Tri[i].normal.y = ny;
				pMesh->triangleList[triCount-1].Tri[i].normal.z = nz;
				pMesh->triangleList[triCount-1].Tri[i].tu = tu;				// uvǥ
				pMesh->triangleList[triCount-1].Tri[i].tv = -tv;
				vCount++;

				temp = pMesh->triangleList[triCount-1].Tri[i].pos - m_ModelList[0].boneList[index].m_pos;
				
				pMesh->triangleList[triCount-1].Tri[i].pos    = VectorTransform( temp, m_ModelList[0].boneList[index].iTM );
				pMesh->triangleList[triCount-1].Tri[i].normal = VectorTransform( temp, m_ModelList[0].boneList[index].iTM );
				
				D3DXVec3Normalize( &pMesh->triangleList[triCount-1].Tri[i].normal, &pMesh->triangleList[triCount-1].Tri[i].normal );
			}
		}
	}
	pMesh->numVertex   = vCount;
	pMesh->numTriangle = triCount;
}

//- Functions for Parsing AnimationSMD -------------------------------------------------------------------------
void C3PCharacter::ParseAnimation( tANI *pAni )
{
	int boneIndex, frame = 0;
	float x, y, z, rx, ry, rz;
	char str[32];

	while( fgets( line, sizeof(line), fp ) != NULL )
	{
		if( sscanf( line, "%d %f %f %f %f %f %f", &boneIndex, &x, &y, &z, &rx, &ry, &rz ) == 7 )
		{
			pAni->ani_pos[frame][boneIndex].x = x;
			pAni->ani_pos[frame][boneIndex].y = y;
			pAni->ani_pos[frame][boneIndex].z = z;

			pAni->ani_rot[frame][boneIndex].x = rx;
			pAni->ani_rot[frame][boneIndex].y = ry;
			pAni->ani_rot[frame][boneIndex].z = rz;
		}
		else if( sscanf( line, "%s", &str ) )
		{
			if( _stricmp( str, "time" ) == 0 )
			{
				sscanf(line, "%s %d", str, &frame );
				continue;
			}

			if( _stricmp( str, "end") == 0 )
			{
				pAni->totalframe = frame;
				return;
			}
		}
	}

}

void C3PCharacter::InitDeviceObjects ( LPDIRECT3DDEVICE8 pd3dDevice )
{
	m_pd3dDevice = pd3dDevice;

	char* Dir = "Data/Player/";
	char file[256];
	int  index = 0;
	int  i, j;

	for( i = 0; i < m_ModelList[0].meshList[0].numOfTextures; i++ )
	{
		strcpy( file, Dir );
		strcat( file, m_ModelList[0].meshList[0].textureList[i] );
		D3DXCreateTextureFromFile( m_pd3dDevice, file, &g_texture[i] );
	}

	// ؽ  
    if( FAILED( m_pd3dDevice->CreateVertexBuffer( m_ModelList[0].meshList[0].numVertex*sizeof(CVertex3),
                                                  0, D3DFVF_VERTEX3, D3DPOOL_MANAGED , &m_pVB ) ) )
    {
        return ;
    }

	m_pVB->Lock( 0, 0, (BYTE**)&m_pVertex, 0 );

		index = 0;

		for( i = 0; i < m_ModelList[0].meshList[0].numTriangle; i++ )
		{

			for( j = 0; j < 3; j++)
			{
				m_pVertex[index].m_vPos		= m_ModelList[0].meshList[0].triangleList[i].Tri[ ClockIndex[j] ].pos;
				m_pVertex[index].m_vNor		= m_ModelList[0].meshList[0].triangleList[i].Tri[ ClockIndex[j] ].normal;
				m_pVertex[index].m_oTex.tu	= m_ModelList[0].meshList[0].triangleList[i].Tri[ ClockIndex[j] ].tu;
				m_pVertex[index].m_oTex.tv	= m_ModelList[0].meshList[0].triangleList[i].Tri[ ClockIndex[j] ].tv;
				index++;
			}

		}

	m_pVB->Unlock();

}

void C3PCharacter::Update()
{
	tBONE_NODE* pCurrentBone;   //   ؾ ϹǷ
	tANI*		pCurrentAni;
	tMESH*		pCurrentMesh;

	pCurrentMesh = &m_ModelList[0].meshList[0];
	pCurrentBone = &m_ModelList[0].boneList[0];
	pCurrentAni  = &m_ModelList[0].aniList[0];

	int i, j, index=0;
	int frameIndex = 0;

	D3DXVECTOR3 temp, temp2;
	D3DXMATRIX rMat;


	int parent;
	for( i = 0; i < m_ModelList[0].numOfBones; i++)
	{
		parent = pCurrentBone[i].parent;

		if( parent == -1 )
		{
			pCurrentBone[i].TM		= Angle2Matrix( pCurrentAni->ani_rot[frameCount][i] );
			pCurrentBone[i].TM		= pCurrentBone[i].TM * pCurrentMesh->rotate;
			pCurrentBone[i].iTM		= *( D3DXMatrixTranspose( &pCurrentBone[i].iTM, &pCurrentBone[i].TM ) );
			pCurrentBone[i].m_pos	= pCurrentBone[i].pos; // θbone ڸ .
			pCurrentBone[i].m_pos.y = -25; //  ƽ ǥ ̹Ƿ ƽ y  D3D  z ǥ ȴ
			pCurrentBone[i].m_pos.z = -20; //  ƽ ǥ ̹Ƿ ƽ z  D3D  y ǥ ȴ
		}
		else
		{
			rMat					= Angle2Matrix( pCurrentAni->ani_rot[frameCount][i] );
			pCurrentBone[i].TM		= pCurrentBone[parent].TM * rMat;
			rMat					= *( D3DXMatrixTranspose( &rMat, &rMat ) );
			pCurrentBone[i].iTM		= rMat * pCurrentBone[parent].iTM;
			temp					= VectorTransform( pCurrentBone[i].pos, pCurrentBone[parent].TM );
			pCurrentBone[i].m_pos	= temp + pCurrentBone[parent].m_pos;
		}

	}



	int boneIndex;

	m_pVB->Lock( 0, 0, (BYTE**)&m_pVertex, 0 );
	
		index=0;
		for( i=0; i < m_ModelList[0].meshList[0].numTriangle; i++)
		{

			for( j=0; j < 3; j++)
			{
				boneIndex	= pCurrentMesh->triangleList[i].Tri[ ClockIndex[j] ].boneIndex;
				temp		= VectorTransform( pCurrentMesh->triangleList[i].Tri[ ClockIndex[j] ].pos, pCurrentBone[ boneIndex ].TM );
				temp2		= temp + pCurrentBone[ boneIndex ].m_pos;
					
				temp		= VectorTransform( pCurrentMesh->triangleList[i].Tri[ ClockIndex[j] ].normal, pCurrentBone[ boneIndex ].TM );
				D3DXVec3Normalize( &temp, &temp );
					
				m_pVertex[index].m_vPos.x	= temp2.x;
				m_pVertex[index].m_vPos.y	= temp2.z;
				m_pVertex[index].m_vPos.z	= temp2.y;
					
				m_pVertex[index].m_vNor.x	= temp.x;
				m_pVertex[index].m_vNor.y	= temp.z;
				m_pVertex[index].m_vNor.z	= temp.y;

				index++;
			}
		}

    m_pVB->Unlock();



	//- frameCount 
	frameCount++;
	if( frameCount > m_ModelList[0].aniList[0].totalframe ) frameCount = 1;
}


void C3PCharacter::Render( void )
{
	int i;
	m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );

	m_pd3dDevice->SetStreamSource( 0, m_pVB, sizeof(CVertex3) );
	m_pd3dDevice->SetVertexShader( D3DFVF_VERTEX3 );

	for(i=0; i<m_ModelList[0].meshList[0].numTriangle; i++)
	{
		m_pd3dDevice->SetTexture( 0, g_texture[ m_ModelList[0].meshList[0].triangleList[i].textureIndex ] );
		m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, i*3, 1 );
	}

}



