#ifndef _CHARACTER_H
#define _CHARACTER_H


#include <D3DX8.h>
#include "Vertex.h"


#define MAXBONE     100
#define MAXFRAME    100
#define MAXANI      100

#define MAXMODEL    10
#define MAXMESH     5
#define MAXTRIANGLE 2048
#define MAXTEXTURE  10


#ifndef SAFE_RELEASE
#define SAFE_RELEASE(p)  { if(p) {  (p)->Release(); (p) = NULL; } }
#endif


#define ANGLEVECTOR D3DXVECTOR3 // rx, ry, rx, 0  





struct tVERTEX
{
	int			 boneIndex;
	D3DXVECTOR3  pos;
	D3DXVECTOR3  normal;
	float		 tu, tv;
	bool		 tflag;
};

struct tTRIANGLE
{
	int			textureIndex;
	tVERTEX		Tri[3];
};

struct tBONE_NODE //  ü
{
	char		node_name[32];  //  ̸
	int			parent;         // θ ε
	D3DXVECTOR3	pos;            //   ġ
	ANGLEVECTOR rot;			//  ȸ
	D3DXVECTOR3 m_pos;			

	D3DXMATRIX	TM;
	D3DXMATRIX  iTM;
};


struct tMESH
{
	int			numVertex;
	int			numTriangle;
	int			numOfTextures;

	D3DXVECTOR3	pos;
	D3DXVECTOR3 rot;
	
	D3DXMATRIX  rotate;

	tTRIANGLE	triangleList[MAXTRIANGLE];
	char		textureList[MAXTEXTURE][128];

	tMESH();
};


struct tANI
{
	char ani_name[32];
	int	 totalframe;

	D3DXVECTOR3 ani_pos[MAXFRAME][MAXBONE];
	ANGLEVECTOR ani_rot[MAXFRAME][MAXBONE];
};

struct tMODEL
{
	int			numOfMeshes;
	int			numOfBones;
	int			numOfAnies;
	tMESH		meshList[MAXMESH];
	tBONE_NODE  boneList[MAXBONE];
	tANI		aniList[MAXANI];
};


class C3PCharacter
{
public:

	LPDIRECT3DDEVICE8			m_pd3dDevice;
	LPDIRECT3DVERTEXBUFFER8		m_pVB;
	
	CVertex3*			        m_pVertex;
	
	char						m_cName[32];
	int							m_iNumModel;
	tMODEL						m_ModelList[MAXMODEL];
	D3DXVECTOR3					m_Positions[MAXMODEL];  // 𵨵  ġ Ʈ
	D3DXVECTOR3					m_Rotations[MAXMODEL];  // 𵨵  ȸ Ʈ

	C3PCharacter();
	~C3PCharacter();


	void InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice);
	void Update();
	void Render();
	void Close();

	bool LoadRefSMD(char* fileName);
	bool LoadAniSMD(char* fileName);

private:

	void ParseNodes( tBONE_NODE *pNodes );
	void ParseSkeleton( tBONE_NODE *pNodes);
	void ParseTriangle( tMESH *pMesh);
	void ParseAnimation( tANI *pAni);
	void SetMatrix( tBONE_NODE *pNodes);
};


#endif